@@ -70,13 +70,20 @@ test_that("binomial regression and stuff works", {
   expect_equal(LogitFitCoef, LogitFit2$coefficients)
   
   ### Branch and bound variable selection with logistic regression
+  ### Checking that each branch and method gives same results
   LogitVS <- VariableSelection(LogitFit, type = "branch and bound")
   LogitVS2 <- VariableSelection(supp ~ ., data = Data, family = "binomial", 
-                                 link = "logit", type = "branch and bound", 
-                                 parallel = TRUE, nthreads = 1)
+                                 link = "logit", type = "backward branch and bound", 
+                                 parallel = TRUE)
   
   expect_equal(LogitVS$finalmodel$coefficients, LogitVS2$finalmodel$coefficients)
   
+  LogitVS3<- VariableSelection(supp ~ ., data = Data, family = "binomial", 
+                                link = "logit", type = "switch branch and bound", 
+                                parallel = TRUE)
+  
+  expect_equal(LogitVS$finalmodel$coefficients, LogitVS3$finalmodel$coefficients)
+  
   ### Forward variable selection with logistic regression
   LogitVS <- VariableSelection(LogitFit, type = "forward")
   LogitVS2 <- VariableSelection(supp ~ ., data = Data, family = "binomial", 
@@ -179,7 +186,8 @@ test_that("poisson regression works", {
   
   ## Fitting poisson regression
   expect_equal(BranchGLM(y ~ ., data = Data, family = "poisson", link = "log", parallel = TRUE)$coefficients, 
-               BranchGLM(y ~ ., data = Data, family = "poisson", link = "log")$coefficients)
+               BranchGLM(y ~ ., data = Data, family = "poisson", link = "log")$coefficients, 
+               tolerance = .Machine$double.eps^(.25))
   
   ## Checking variable selection
   ### branch and bound
